/**
 * 
 */
package com.lombardisoftware.gwtcommon.client;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.FlowPanel;

import com.lombardisoftware.gwtcommon.client.PipesMenu.ActionLink;
import com.lombardisoftware.gwtcommon.client.clientObjects.AbstractClientObject;

/**
 * An AbstractContentPanel is a container for a collection of panels which all appear under the
 * same top-level tab. It provides a mechanism for switching between those pages when an action
 * is taken -- a link is clicked, an object selected, etc.
 * 
 * Also provides some structure for supporting changing panels when an object is selected.
 * 
 * @author jsugg
 *
 */
public abstract class AbstractContainerPanel extends FlowPanel {
    
    public static final String LIST_PANEL_KEY = "list panel key";
    public static final String DETAILS_PANEL_KEY = "details panel key";

    // a list of ActionLink objects
    private List<ActionLink> crumbs = new ArrayList<ActionLink>();

    // list of listeners for changes to the current selection
    private List<ISelectionListener> selectionListeners = new ArrayList<ISelectionListener>();
    
    private Map<String, AbstractRepositoryPanel> panelMap = null;
    private String current = null;
    private String newPanel = null;

    public AbstractContainerPanel() {
        super();

        setStyleName("twMainPanel");
        
        // initialize the panel map in the sub-class
        panelMap = initPanels();
        
        // initialize the current panel to the projects panel
        current = getDefaultPanelKey();
    }
    
    protected String getDefaultPanelKey() {
        return LIST_PANEL_KEY;
    }

    /**
     * Allow sub-class to build the appropriate panels with mappings they know about.
     * @return   the map of string names to panels
     */
    protected abstract Map<String, AbstractRepositoryPanel> initPanels();
    
    public final void setCurrent(String panelKey) {
        // only set the current if the panelMap actually contains the specified key
        if (!panelMap.containsKey(panelKey))
            return;

        final boolean isFirstTime = getWidgetCount() == 0;
        final boolean oldIsNew = panelKey.equals(current);
        if(oldIsNew && !isFirstTime) {
            getCurrentPanel().refresh();
            return;
        }

        final AbstractRepositoryPanel oldPanel = getCurrentPanel();
        if(!oldIsNew && !isFirstTime) {
            CommonUtils.setPosition(oldPanel, CommonUtils.StylePosition.Absolute);
        }
        current = panelKey;
        final AbstractRepositoryPanel newPanel = getCurrentPanel();
        newPanel.refresh();
        newPanel.setOpacity(0);
        add(newPanel);

        FadeAnimation fadeInNewPanelAnimation = new FadeAnimation(newPanel);
        fadeInNewPanelAnimation.fadeTo(1.0, CommonUtils.TRANSITION_ANIMATION_INTERVAL, new Command() {
            public void execute() {
                if(!oldIsNew && !isFirstTime) {
                    remove(oldPanel);
                    CommonUtils.setPosition(oldPanel, CommonUtils.StylePosition.Relative);
                }
            }
        });

        this.newPanel = null;
    }

    public void refresh() {
        if(newPanel != null) {
            setCurrent(newPanel);
        } else {
            setCurrent(current);
        }
    }

    public void refreshContents() {
        getCurrentPanel().refreshContents();
    }
    
    public void addSelectionListener(ISelectionListener listener) {
        if (!selectionListeners.contains(listener))
            selectionListeners.add(listener);
    }
    
    public void removeSelectionListener(ISelectionListener listener) {
        if (selectionListeners.contains(listener))
            selectionListeners.remove(listener);
    }
    
    public void setSelectedObject(AbstractClientObject theObject) {
        // notify any interested parties that our selection has changed
        // any listeners who want to refresh at this point need to call setCurrent
        for (ISelectionListener listener : selectionListeners) {
            listener.selectionChanged(theObject);
        }
    }

    public void addCrumb(ActionLink action) {
        crumbs.add(action);
    }
    
    public List<ActionLink> getCrumbs() {
        // should we be returning a clone here?
        return crumbs;
    }
    
    protected AbstractRepositoryPanel getCurrentPanel() {
        return panelMap.get(current);
    }

    public void goToPage(List<String> tokens, AsyncCallback callback) {
        if (tokens.size() == 0) {
            return;
        }
        
        String token = tokens.get(0);
        if (panelMap.containsKey(token)) {
            AbstractRepositoryPanel panel = panelMap.get(token);
            newPanel = token;
            if(tokens.size() > 1) {
                panel.goToPage(CommonUtils.subList(tokens, 1, tokens.size()), callback);
            }
            else {
                callback.onSuccess(this);
            }
            // we rely on the parent panel to refresh in the callback
        }
        else {
            Window.alert("AbstractContainerPanel.goToPage token: " + token); // $NON-NLS-1$
            Window.alert(CommonUtils.getCommonMessages().getString("unableToNavigateError"));
        }
    }

    /**
     * Sets panel opacity.
     * @param opacity double Opacity level.
     */
    public void setOpacity(final double opacity) {
        this.getElement().getStyle().setProperty("opacity", "" + opacity);
    }
}
