/**
 * 
 */
package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.InvocationException;
import com.google.gwt.user.client.ui.FormHandler;
import com.google.gwt.user.client.ui.FormPanel;
import com.google.gwt.user.client.ui.FormSubmitCompleteEvent;
import com.google.gwt.user.client.ui.FormSubmitEvent;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * Abstract handler for all GWT TeamWorks callbacks -- handles exceptional cases in a uniform way.
 * Lost contact, reauthentication, etc.
 * 
 * @author jsugg
 *
 */
public abstract class TWCallbackCommand<T> implements AsyncCallback<T>, Command {

    private final String XML_REQUEST_ERROR_STRING = "XmlHttpRequest.status";

    final public void onFailure(Throwable caught) {
        if (isLoginError(caught)) {
            handleLoginError();
        } else if (caught instanceof InvocationException || isRuntimeXMLRequestError(caught)) {
            handleConnectionException(caught);
        } else {
            handleFailure(caught);
        }
    }

    private void handleLoginError() {
        
        if (!CommonUtils.isRunningInAE()) {
            CommonUtils.invalidateSession();
        } else {            
            loginUsingAECredentials();
        }
    }

    private void loginUsingAECredentials() {
        
        final FormPanel form = new FormPanel();
        RootPanel.get().add(form);
        
        form.setMethod(FormPanel.METHOD_POST);
        form.setAction("j_security_check");
        VerticalPanel formContents = new VerticalPanel();
        form.setWidget(formContents);
        form.setVisible(false);
        
        // now add the inputs
        TextBox userText = new TextBox();
        formContents.add(userText);
        userText.setName("j_username");
        String userName = CommonUtils.getLoginUser();
        userText.setText(userName);

        PasswordTextBox pwText = new PasswordTextBox();
        formContents.add(pwText);
        pwText.setName("j_password");
        String loginPassword = CommonUtils.getLoginPassword();
        pwText.setText(loginPassword);

        form.addFormHandler(new FormHandler() {
            public void onSubmit(FormSubmitEvent event) {
                // do nothing
            }
            public void onSubmitComplete(FormSubmitCompleteEvent event) {
                RootPanel.get().remove(form);
                ExceptionPopup.closeException();
            }
        });
        RootPanel.get().add(form);
        form.submit();
        
    }

    private boolean isLoginError(Throwable caught) {
        // LOGIN keyword is generated by login.jsp of process-admin and repository 
        // check also whether login is part of the error to handle session time-out  ||
        return ( caught.getMessage().indexOf("ProcessAdmin/login")>=0 ||
                 caught.getMessage().indexOf("ProcessCenter/login")>=0 || 
                 "LOGIN".equals(caught.getMessage().trim()));        
    }

    protected void handleConnectionException(Throwable caught) {
        ExceptionPopup.showException(caught, CommonUtils.getCommonMessages().getString("errorServerUnreachable"));
    }

    public abstract void onSuccess(T result);

    protected void handleFailure(Throwable caught) {
        ExceptionPopup.showException(caught, getFailureMessage(caught));
    }
    
    protected String getFailureMessage(Throwable caught) {
        return caught.getMessage();
    }
    
    public abstract void execute();

    /**
     * Method checks for network connection error which is not InvocationError but RuntimeException
     * caught at XmlRequest processing stage with message like:
     * XmlHttpRequest.status == undefined, please see Safari bug...
     * or
     * Unable to read XmlHttpRequest.status; likely causes are a...
     *
     * @param ex Exception to check.
     * @return True if it's known connection error. False if other type of error.
     * @see com.google.gwt.http.client.XMLHTTPRequest#getBrowserSpecificFailure
     */
    private boolean isRuntimeXMLRequestError(Throwable ex) {
        if (ex instanceof RuntimeException && ex.getMessage().contains(XML_REQUEST_ERROR_STRING)) {
            return true;
        }
        return false;
    }
}
