package com.lombardisoftware.gwtcommon.client.preferences;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.Widget;
import com.lombardisoftware.gwtcommon.client.CommonUtils;

import java.util.ListIterator;

import static com.lombardisoftware.gwtcommon.client.preferences.UserPreferences.*;


/**
 *
 * Pop-up window showing user preferences i.e. language prefs.
 *
 * By default is aligns to the right.
 *
 *
 * @author Slava Nadvorny
 */
public class PreferencesPopup extends PopupPanel {

    private static final int PROPERTY_ROW_NUMBER = 1;
    private Button saveButton;
    private UserPreferences prefs;

    private static final String USER_PREFERENCES = CommonUtils.getCommonMessages().getString("userPref");
    private static final String LOCALE_PREFERENCES = CommonUtils.getCommonMessages().getString("localPref");
    private static final String SAVE_BUTTON_LABEL = CommonUtils.getCommonMessages().getString("saveChanges");
    private static final String CLOSE_PREFERENCES_POP_UP = CommonUtils.getCommonMessages().getString("closePopup");
    private static final String YOU_NEED_TO_RELOGIN = "* "+CommonUtils.getCommonMessages().getString("reloginOrder");
    private ListBox localeDropDown;


    /**
     * Creates user prefs pop-up for Process Admin and Process Center.
     * 
     * @param prefs UserPreferences provider.
     */
    public PreferencesPopup(final UserPreferences prefs) {
        super(true , true);     // Make pop-up auto-hide and modal.

        this.prefs = prefs;

        setStyleName("prefsPopup");

        FlowPanel mainPanel = new FlowPanel();

        // Close button.
        FlowPanel close = new FlowPanel() {
            {
                sinkEvents(Event.ONCLICK);
            }

            public void onBrowserEvent(Event event) {
                super.onBrowserEvent(event);
                int type = DOM.eventGetType(event);
                if (type == Event.ONCLICK) {
                    PreferencesPopup.this.hide();
                }
            }
        };
        
        close.setTitle(CLOSE_PREFERENCES_POP_UP);
        mainPanel.add(close);
        close.addStyleName("closeButton");

        // Pop-up title.
        Label prefsTitleLabel = new Label(USER_PREFERENCES);
        prefsTitleLabel.setStyleName("prefsTitle");
        mainPanel.add(prefsTitleLabel);

        // Prefs grid.
        Grid prefsGrid = new Grid(PROPERTY_ROW_NUMBER, 2);
        prefsGrid.setStyleName("prefsGrid");

        // Locale preference label.
        Label localePrefLabel = new Label(LOCALE_PREFERENCES);
        prefsGrid.setWidget(0, 0, localePrefLabel);

        // Locale preference drop-down.
        localeDropDown = new ListBox(false);

        localeDropDown.addChangeListener(new ChangeListener() {
            public void onChange(Widget sender) {
                onLocaleChanged();
            }
        });

        prefsGrid.setWidget(0, 1, localeDropDown);
        mainPanel.add(prefsGrid);

        // Hint.
        Label hint = new Label(YOU_NEED_TO_RELOGIN);
        hint.setStyleName("localePrefHint");
        mainPanel.add(hint);

        // Save button wrapped into DIV.
        FlowPanel buttonWrapper = new FlowPanel();
        buttonWrapper.setStyleName("buttonWrapper");
        mainPanel.add(buttonWrapper);

        saveButton = new Button(SAVE_BUTTON_LABEL);
        saveButton.setStyleName("savePrefsButton");
        saveButton.setEnabled(false);
        buttonWrapper.add(saveButton);

        saveButton.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                onSaveClick();
            }
        });

        add(mainPanel);
    }

    private void reset() {
        saveButton.setEnabled(false);
        localeDropDown.clear();
    }

    private void init() {
        prefs.loadPreferences(new AsyncCallback<PreferencesDataObject>() {
            public void onFailure(Throwable caught) {
                GWT.log("Can't load list of server supported locales", caught);
            }

            public void onSuccess(PreferencesDataObject result) {
                prefs.setData(result);
                ListIterator<DisplayLocale> localeIter = result.getSupportedLocales().listIterator();

                while(localeIter.hasNext()) {
                    int localeIndex = localeIter.nextIndex();
                    DisplayLocale locale = localeIter.next();

                    localeDropDown.addItem(locale.getDisplayName(), locale.getLocale());

                    if (locale.getLocale().equals(result.getPreferenceValue(PropertyName.LOCALE))) {
                        localeDropDown.setSelectedIndex(localeIndex);
                    }
                }
            }
        });

    }

    private void onLocaleChanged() {
        int selectedIndex = localeDropDown.getSelectedIndex();
        prefs.setPreference(PropertyName.LOCALE, localeDropDown.getValue(selectedIndex));
        allowSave();
    }

    private void onSaveClick() {
        prefs.storePreferences(new AsyncCallback<Void>() {
            
            public void onFailure(Throwable caught) {
                GWT.log("Could not save user preferences", caught);
                PreferencesPopup.this.hide();
            }

            public void onSuccess(Void result) {
                PreferencesPopup.this.hide();
            }

        });
        saveButton.setEnabled(false);
    }

    /**
     * Enable save button and disable auto-hide on pop-up.
     */
    private void allowSave() {
        // setAutoHideEnabled(false);       // Enable in GWT 2.0
        saveButton.setEnabled(prefs.hasChanges());
    }

    /**
     * Show aligns pop-up to the right.
     */
    @Override
    public void show() {
        init();
        
        // pop-up top left corner is 300px left from right border of page. 
        int left = (Window.getClientWidth() - (getOffsetWidth() + 300));
        int top = 30;
        setPopupPosition(
                Math.max(Window.getScrollLeft() + left, 0),
                Math.max(Window.getScrollTop() + top, 0)
        );

        super.show();
    }

    /**
     * Hiding pop-up also resets prefs to current.
     * @param autoClosed
     */
    @Override
    public void hide(boolean autoClosed) {
        reset();
        super.hide(autoClosed);
    }
}
