// BEGIN COPYRIGHT
// *************************************************************************
//
//  Licensed Materials - Property of IBM
//  5725-A09
//  Copyright IBM Corporation 2010. All Rights Reserved.
//  US Government Users Restricted Rights- Use, duplication or disclosure
//  restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

package com.lombardisoftware.gwtcommon.server;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Category;
import org.apache.log4j.Logger;

import com.lombardisoftware.client.SortUtils;
import com.lombardisoftware.client.common.ListResult;
import com.lombardisoftware.client.delegate.SecurityDelegate;
import com.lombardisoftware.client.delegate.SecurityDelegateFactory;
import com.lombardisoftware.client.persistence.UserGroup;
import com.lombardisoftware.client.persistence.common.ID;
import com.lombardisoftware.client.persistence.common.POType;
import com.lombardisoftware.core.TeamWorksException;
import com.lombardisoftware.core.UserInfo;
import com.lombardisoftware.gwtcommon.client.clientObjects.ClientPrincipal;
import com.lombardisoftware.gwtcommon.client.clientObjects.ClientPrincipal.Type;
import com.lombardisoftware.gwtcommon.client.clientObjects.ComplexResult;
import com.lombardisoftware.server.ejb.persistence.SortedFilter;

/**
 * @author Slava Nadvorny
 */
public class CommonService {
    
    private static final Category LOGGER = Logger.getLogger(CommonService.class);

    public static final int MAX_PRINCIPALS = 200;
    public static final int DEFAULT_PRINCIPALS = 20;

    public static ComplexResult<ClientPrincipal> getPrincipals(ClientPrincipal principal, int start, int maxItems, SortedFilter sortedFilter) {
        
        ComplexResult<ClientPrincipal> principals = new ComplexResult<ClientPrincipal>();

        String filter = principal.getName();

        if(LOGGER.isDebugEnabled()){
        	LOGGER.debug("getPrincipals: principalType="+principal.getType()+", start="+maxItems+", maxItems="+maxItems+", filter="+filter);
        }
        // if maxItems is > MAX_PRINCIPALS we limit the query to MAX_PRINCIPALS
        if(maxItems <=0 || maxItems > MAX_PRINCIPALS)
        	maxItems = MAX_PRINCIPALS;
        try {
            if (principal.getType() == null || principal.getType() == Type.USER) {
                
                findUsers(principal, maxItems, principals, filter);
            }

            // If result has less than maxItems results fill in the rest with groups.
            if (principals.size() < maxItems          
                    && (principal.getType() == null || principal.getType() == Type.GROUP)) {

                findGroups(principal, maxItems - principals.size(), principals, filter);
            }

            SortUtils.sort(principals.getList(),
                            new SortUtils.FieldSelector<ClientPrincipal>() {
                                public String getValue(ClientPrincipal item) {
                                    return item.getFullName() != null ? item.getFullName():item.getName();
                                }
                            });

        } catch (TeamWorksException e) {
            LOGGER.error("Failed to find principals", e);
        }

        return principals;
    }

    private static void findGroups(ClientPrincipal principal, int maxItems, ComplexResult<ClientPrincipal> principals, String filter) throws TeamWorksException {
        
        if (principal.getName() != null && principal.getName().trim().length() > 0) {
            String groupFilter = filter.replace("%","").replace("*","");
            // Maximum number of groups to return.
            int maxGroupsNumber = maxItems < MAX_PRINCIPALS? maxItems : MAX_PRINCIPALS;
            
            ListResult<UserGroup> userGroups = getSecurityDelegate()
                    .listGroupsWithFilter("*" + groupFilter + "*", maxGroupsNumber);
            
            for(UserGroup group: userGroups.getList()) {
                principals.add(fromGroup(group));
            }
            if (userGroups.hasMore()) {
                principals.setHasMore(true);
            }
        } else {
            List<UserGroup> userGroups = getSecurityDelegate().listGroups(
                    new com.lombardisoftware.server.ejb.persistence.Filter(){});
            for (UserGroup group : userGroups) {
                principals.add(fromGroup(group));
            }
        }
    }

    // Searching Users List is restricted to 20 users in UI.
    /*
     * findUsers
     * 		Find list of users matching the filter and fill up the prinicpals
     * 
     * @principals: in-out parameter
     */
    private static void findUsers(ClientPrincipal principal, int maxItems, ComplexResult<ClientPrincipal> principals, String filter) throws TeamWorksException {
        // Both "*" and "%" should work as wildcard indicators for user and group search.
        
        // Append a * to the end of the prefix so that we get everything that starts with the name
        String refinedFilter = principal.getName() == null ? "*" : ("*" + filter + "*");

        int limit = maxItems < MAX_PRINCIPALS ? maxItems : MAX_PRINCIPALS;
        // Find limit + 1 to notify user if there are more users available then limit. 
        List<String> userNames = getSecurityDelegate().listUserNamesWithFilter(refinedFilter, limit + 1);

        if (principal.getName() != null && principal.getName().trim().length() > 0) {
            String userFilter = filter.replace("%","").replace("*","");
            refinedFilter = "*" + userFilter + "*";
        }
        else { 
            refinedFilter = "*";
        }
      

       if (userNames.size() > limit) {
            userNames = new ArrayList<String>(userNames.subList(0, limit));
            principals.setHasMore(true);    
        }

        List<UserInfo> users = getSecurityDelegate().getOrCreateUsersByNames(userNames);

        for(UserInfo userInfo: users) {
            principals.add(fromUser(userInfo));
        }
    }

    private static SecurityDelegate getSecurityDelegate() {
        return SecurityDelegateFactory.getInstance().newInstance();
    }
    
    private static ClientPrincipal fromUser(UserInfo user) throws TeamWorksException {
        return new ClientPrincipal(
                Type.USER,
                ID.toExternalString(user.getUserId()),
                user.getUsername(),
                user.getFullname(),
                ""
        );
    }

    private static ClientPrincipal fromGroup(UserGroup group) throws TeamWorksException {
        ClientPrincipal principal = new ClientPrincipal(
                Type.GROUP,
                ID.toExternalString(group.getGroupId()),
                group.getName(),
                group.getName(),
                group.getDescription(),
                0,
                !Integer.valueOf(UserGroup.GROUP_TYPE_INTERNAL).equals(group.getGroupType())
        );
        ID<POType.UserGroup> parentGroupId = group.getParentGroupId();
        principal.setTeamManagerGroupId(ID.toExternalString(parentGroupId));
        if(parentGroupId != null) {
            UserGroup parentGroup = getSecurityDelegate().getGroup(parentGroupId);
            principal.setTeamManagerGroupName(parentGroup!=null ? parentGroup.getName() : "");
        }
        return principal;
    }
    
    
}
