package com.lombardisoftware.gwtcommon.server.util;

import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;

public abstract class LocalizationUtils {

	public final static String DEFAULT_CHARSET = "UTF-8";
	public final static String DEFAULT_LOCALE = "en_US";
	public final static String MISSING_RESOURCE_BUNDLE_CONTENT = "{\"__MISSING__\": \"__MISSING__\"}";

	public static String getResourceBundleContent(String resourceBundleRootPath,
			String resourceBundleName, String locale) throws IOException {
		
		String content = null;
		if (locale == null || locale.length() == 0) {
			locale = DEFAULT_LOCALE;
		}
		if (!resourceBundleRootPath.endsWith("/")) {
			resourceBundleRootPath = resourceBundleRootPath + "/";
		}
		InputStream stream = getResourceBundleContentImpl(resourceBundleRootPath, resourceBundleName, locale);
		if (stream == null && !locale.equals(DEFAULT_LOCALE)) {
			// try to locale and load default resource bundle
			stream = getResourceBundleContentImpl(resourceBundleRootPath, resourceBundleName, DEFAULT_LOCALE);
			if (stream == null) {
				// try to locale and load fallback (without locale suffix) resource bundle
				stream = getResourceBundleContentImpl(resourceBundleRootPath, resourceBundleName, null);
			}
		}
		if (stream != null) {
			content = readAll(new InputStreamReader(stream, DEFAULT_CHARSET));
			safeClose(stream);
		} else {
			content = MISSING_RESOURCE_BUNDLE_CONTENT;
		}
		return content;
	}

	private static InputStream getResourceBundleContentImpl(String resourceBundleRootPath,
			String resourceBundleName, String locale) {
		
		InputStream stream = getResourceInputStream(resourceBundleRootPath, resourceBundleName, locale);
		if (stream == null) {
			String extension = stripUnderscore(locale);
			if (extension == null) {
				return null;
			}
			stream = getResourceBundleContentImpl(resourceBundleRootPath, resourceBundleName, extension);
		}
		return stream;
	}

	private static InputStream getResourceInputStream(String resourceBundleRootPath,
			String resourceBundleName, String locale) {
		
		String path = resourceBundleRootPath + resourceBundleName;
		if (locale != null) {
			path = path + "_" + locale;
		}
		path = path + ".json";
		return LocalizationUtils.class.getResourceAsStream(path);
	}

	private static String stripUnderscore(String name) {
		int index = name.lastIndexOf('_');
		if (index != -1) {
			return name.substring(0, index);
		}
		return null;
	}

	/**
	 * Read all of the content and close the reader.
	 * 
	 * @param in
	 * @return
	 * @throws IOException
	 */
	private static String readAll(Reader in) throws IOException {
		try {
			StringWriter out = new StringWriter();
			copy(in, out);
			return out.toString();
		}
		finally {
			safeClose(in);
		}
	}

	/**
	 * Copy the contents of the in reader into the out writer.  The caller is responsible
	 * for closing the streams.
	 * 
	 * @param in
	 * @param out
	 * @return A count of the chars copied
	 * @throws IOException
	 */
	private static int copy(Reader in, Writer out) throws IOException {
		char[] buf = new char[4096];
		int num;
		int off = 0; 
		int total = 0;
		while ((num = in.read(buf, off, buf.length)) != -1) {
			out.write(buf, off, num);
			total += num;
		}
		return total;
	}

	private static void safeClose(Closeable c) {
		try {
			// A common use for safeClose is invoking it within the finally
			// block where an exception may be thrown.  Avoid replacing the
			// active exception with an NPE.
			if (null != c) {
				c.close();
			}
		} catch (IOException ex) {
			// ignore
		}
	}
}
