/*
 * Copyright (C) 2009 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such 
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into 
 * with Lombardi Software. No rights or licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement. 
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*  
 * Coach input and output formatting library.
 *  is a localization wrapper for number-functions.js.
 *  see: http://www.xaprb.com/blog/2006/01/05/javascript-number-formatting/
 *
 * usage:
 *       var output = Formatter("fr").format("12345,67890","###,###,###.##");
 *       var output = Formatter("fr", "en").format("12345.67890","###,###,###.##"); 
 */
/**
 * Create a new Formatter
 * @param lang the output locale
 * @param inputLang the input locale
 *
 * @class
 * @constructor
 */
var Formatter = function(lang, inputLang) {
    
    if (typeof lang == "undefined") {
        lang = "en";
        inputLang = "en";
    } else {
        if (typeof inputLang == "undefined") {
            inputLang = lang;
        }
    }
    this.initDelimiters(lang, inputLang);
}
/**
 * Format the given value using the given format pattern
 * @param input the input value to be formatted
 * @param format the format pattern to be used
 * @return the formatted value
 */
Formatter.prototype.format = function(input, format) {
    
    if (input == null) { 
        return null;
    }
    if (typeof input == "undefined") {
        return undefined;
    }
    if (input == "") {
        return "";
    }
    var normalizedInput = this.delocalizeInput(input);
    var formattedOutput = Formatter.formatNumber(normalizedInput, format);
    var localizedOutput = null;
    if (formattedOutput != undefined) {
        localizedOutput = this.localizeOutput(formattedOutput);
    } else { // the input value is invalid
        // fallback to original value
        localizedOutput = input;
    }
    
    return localizedOutput;
};
/**
 * Format the given value as number using the given format pattern
 * @param input the input value to be formatted
 * @param format the format pattern to be used
 * @return the formatted value
 *
 * @static
 */
Formatter.formatNumber = function(input, format) {
    format = format.replace(/([^0#])[.]([^0#])/g, "$1|$2"); // protect dots in format
    var formattedOutput = new NumberFormatter().format(input,format);;
    return formattedOutput;
}
/**
 * Remove all alphanumeric symbols from format
 * @param val the format to be normalized
 * @return the normalized format
 */
Formatter.prototype.cleanUp = function(val) {
    val = val.replace(/[^0-9.Ee-]/g, ""); // "[eE]" are for scientific format
    val = val.replace(/(\d)-/g,"$1"); // replace all '-' inside numbers
    return val;
}
/**
 * Remove trailing spaces 
 * @param value the string to be trimmed
 * @return the trimmed value
 */
Formatter.prototype.trim = function (value) {
    return value.replace(/^\s*/, "").replace(/\s*$/, "");
}
/**
 * Initialize decimal format symbols (delimiters)
 * @param outputLang the output locale
 * @param inputLang the input locale
 */
Formatter.prototype.initDelimiters = function(outputLang, inputLang) {
    var langTable = {
            en: {dec: ".", ths: ","},
            no: {dec: ",", ths: "."},
            de: {dec: ",", ths: "."},
            fr: {dec: ",", ths: "."},
            nl: {dec: ",", ths: "."},
            it: {dec: ",", ths: "."},
            sv: {dec: ",", ths: "."},
            pt: {dec: ",", ths: "."},
            es: {dec: ",", ths: "."},
            ru: {dec: ",", ths: " "},
            ch: {dec: ".", ths: "'"},  // Switzeland
            pt_BR: {dec: ",", ths: "."},
            el: {dec: ",", ths: "."}
    };
    
    langTable["uk"] = langTable["ru"]; // synonyms
    
    if (langTable[inputLang] == null) {
        inputLang = "en"; // fallback to US locale by default
    }
    
    if (langTable[outputLang] == null) {
        outputLang = "en"; // fallback to US locale by default
    }
    
    this.inputDecimal = langTable[inputLang]["dec"];
    this.inputThousands = langTable[inputLang]["ths"];
    
    this.outputDecimal = langTable[outputLang]["dec"];
    this.outputThousands = langTable[outputLang]["ths"];
}
/**
 * Apply localization to a number
 * @param output the value to be localized
 * @return the localized value
 */
Formatter.prototype.localizeOutput = function(output) {
    
    output = output.replace(/[|]/g, "."); // restore protected dots from format
    
    output = output.replace(/(.*\d)[.](?=\d)/, "$1|"); // last dot between numbers is decimal separator
    output = output.replace(/[,]/g, "~");
    
    output = output.replace(/[|]/, this.outputDecimal);
    output = output.replace(/[~]/g, this.outputThousands);
    
    return output;
}
/**
 * Remove localization from a number, convert to generic JS Number format
 * @param input the value to be delocalized
 * @return the delocalized value
 */
Formatter.prototype.delocalizeInput = function(input) {
        
    input = input.replace(new RegExp("(.*)[" + this.inputDecimal + "]"), "$1|");
    input = input.replace(new RegExp("[" + this.inputThousands + "]", "g"), "");
    input = input.replace(/[|]/, ".");
    return this.cleanUp(input);
}
/**
 * Remove localization from a number, convert to generic JS Number format. Alphanumeric symbols
 * except for the trailing spaces are not removed in order to validate the field. 
 * This function therefore should be called before validating the field value
 * @param input the value to be delocalized
 * @param format format of the given field. Special symbols from format should be silently eliminated before validation
 * @return the delocalized value
 */
Formatter.prototype.delocalizeInputWithoutCleanup = function(input, format) {
        
    input = input.replace(new RegExp("(.*)[" + this.inputDecimal + "]"), "$1|");
    input = input.replace(new RegExp("[" + this.inputThousands + "]", "g"), "");
    input = input.replace(/[|]/, ".");
    // Trimming trailing spaces
    input = this.trim(input);
    // Special formatting symbols (e.g. currency) should be eliminated before validation
    if (format != null) {
        input = Formatter.eliminateFormattingSymbols(input, format, true);
    }
    return input;
}
function isDigit(symbol) {
        if (symbol.length > 1)
                return false;
        var digits = "1234567890";
        if (digits.indexOf(symbol) != -1)
                return true;
        return false;
}
/**
 * Eliminate symbols from a field which are found in a format. Commas, dots, digits and # symbols are ignored.
 * Required for correct work with currency fields.
 * @param input value of the field
 * @param format given format
 * @return cleaned up value
 */
Formatter.eliminateFormattingSymbols = function(input, format, isNumeric) {
        for (var i = 0; i < format.length; i++) {
                var symbol = format.charAt(i);
                if ((isNumeric == true && (symbol != ',' && symbol != '.' && symbol != '#' && isDigit(symbol) == false)) ||
                                (isNumeric == false && symbol != '#')) {                                 
                        var position = input.indexOf(symbol);
                        // we should skip negative sign
                        if (symbol == '-' && position == 0)
                                position = input.indexOf(symbol, position + 1);
                        if (position != -1) {
                                // It is inconvenient to use regexp below because of special symblols (e.g. $)
                                input = input.substring(0, position) + input.substring(position + 1, input.length);
                        }
                }
        }
        return input;
}
/**
 * Remove all formatting and delocalize input fields before submit
 *
 * @static
 */
Formatter.prototype.toNumbers = function() {
    var inputs = document.getElementsByTagName('input');
    for(var i = 0; i<inputs.length; i++) {
        Formatter.parseInput(inputs[i]);
    }
}
/**
 * Remove formatting and delocalize the given input field
 * @param field the field which to parse value for
 *
 * @static
 */
Formatter.parseInput = function(field) {
    if (field.getAttribute("type") == "text" && typeof field.value != "undefined" && field.value != null) {
        // parsing numeric inputs
        if (field.getAttribute("locale") != null
                        && Formatter.isNumericField(field)
            && field.getAttribute('delocalized') != 'true') {
        var lang = field.getAttribute('locale');
        var f = new Formatter(lang);
        field.value = f.delocalizeInputWithoutCleanup(field.value, field.getAttribute('format'));
        field.setAttribute('delocalized', 'true');
    }
        // parsing string inputs
        if (Formatter.isNotNumericField(field) && Formatter.isFormattedField(field)) {
                field.value = Formatter.eliminateFormattingSymbols(field.value, field.getAttribute('format'), false);
}
    }
}
/**
 * Apply formatting to input and output fields at startup
 *
 * @static
 */
Formatter.initFormats = function() {
    var inputs = document.getElementsByTagName('input');
    for(var i = 0; i<inputs.length; i++) {
        Formatter.initInputFormat(inputs[i]);
    }
    
    var outputs = document.getElementsByTagName('p');
    for(var i = 0; i<outputs.length; i++) {
        Formatter.initOutputFormat(outputs[i]);
    }
}
/**
 * Apply formatting to a given input field
 * @param field the field which to format value for
 *
 * @static
 */
Formatter.initInputFormat = function(field) {
    if (field.getAttribute("locale") != null) { 
        if (Formatter.isNumericField(field)) {
        // field is numeric and formatted
                if (Formatter.isFormattedField(field)) {
                field.value =
                        new Formatter(field.getAttribute("locale"), "en")
                        .format(field.value, field.getAttribute("format"));
        } else {
                // field is numeric and not formatted 
                field.value = new Formatter(field.getAttribute("locale")).localizeOutput(field.value);
        }
        } else if (Formatter.isNotNumericField(field)) {
        // field is not numeric
                if (Formatter.isFormattedField(field)) {
                field.value =
                        new Formatter(field.getAttribute("locale"), field.getAttribute("locale"))
                                        .format(field.value, field.getAttribute("format"));
                }
        }
        field.setAttribute('delocalized', null);
    }
}
/**
 * Apply formatting to a given output field
 * @param field the field which to format value for
 *
 * @static
 */
Formatter.initOutputFormat = function(field) {
    if (field.getAttribute("locale") != null
            && field.childNodes[0] != null
            && field.childNodes[0].nodeValue != null) {
        if (Formatter.isNumericField(field)) {
        // field is numeric and formatted
                if (Formatter.isFormattedField(field)) {
                var f = new Formatter(field.getAttribute("locale"), "en");
                field.childNodes[0].nodeValue = f.format(
                                field.childNodes[0].nodeValue,
                                field.getAttribute("format"));
        } else {
                // field is numeric and not formatted
                field.childNodes[0].nodeValue = new Formatter(field.getAttribute("locale")).localizeOutput(field.childNodes[0].nodeValue);
        } 
        } else if (Formatter.isNotNumericField(field)) {
        // field is not numeric
                if (Formatter.isFormattedField(field)) {
                var f = new Formatter(field.getAttribute("locale"), field.getAttribute("locale"));
        field.childNodes[0].nodeValue = f.format(
                                field.childNodes[0].nodeValue,
                                field.getAttribute("format"));
        } 
    }
}
}
/**
 * Check whether the given field is formattable
 * @param field the field to be checked
 *
 * @static
 */
Formatter.isFormattedField = function(field) {
    if (Formatter.getFieldBaseType(field) == "DateTime") return false;  // DateTime variables are skipped
        return field.getAttribute("format") != null;
}
/**
 * Check whether the given field is numeric
 * @param field the field to be checked
 *
 * @static
 */
Formatter.isNumericField = function(field) {
        return field.getAttribute("isNumeric") == "true";
}
/**
 * Check whether the given field is not numeric
 * @param field the field to be checked
 *
 * @static
 */
Formatter.isNotNumericField = function(field) {
        return field.getAttribute("isNumeric") == "false";
}
/**
 * Return base type for the given field
 * @param field the field to be checked
 * @return base type name
 *
 * @static
 */
Formatter.getFieldBaseType = function(field) {
        var baseType = field.getAttribute("baseType");
        return baseType;
}
// execute initFormats on coach load.
addOnPageLoadFunction(Formatter.initFormats);