<?xml version="1.0"?>

<!--
  - This file provides the transformation from a BPD XML to an SVG file.
  -
  - The transformation should operate on the BPD, it may be given a parameter by
  - the name 'annotation-url' which gives a list of halos.  The skin should be
  - provided via parameter 'skin-url'.
  -
  - This file depends upon 'skin.xsl' being in the same directory.
  -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:svg="http://www.w3.org/2000/svg"
	xmlns:lombardi="http://www.lombardi.com/bpd/svg/"
	xmlns:skin="http://www.lombardi.com/bpd/svg/skin"
	xmlns:a3="http://ns.adobe.com/AdobeSVGViewerExtensions/3.0/"
	xmlns:xlink="http://www.w3.org/1999/xlink"
	exclude-result-prefixes="#all"
	version="2.0">

	<!--
	  - Include the skinning file.  Requires parameter 'skin-url', and for the
	  - skin.xsl file to be in the same directory.
	  -->
	<xsl:include href="skin.xsl"/>

	<!--
	  - Ensure we have SVG output.
	  -->
	<xsl:output method="xml" indent="yes" media-type="image/svg" />

	<!--
	  - Include the halo definitions.  This parameter is optional.  If it is not
	  - present or the file cannot be found, then no halos will be used.
	  -->
	<xsl:param name="annotation-url"/>
    <xsl:param name="xml-base"/>
	<!--
	  - Extract the annotation from the url
	  -->
	<!--<xsl:variable name="annotation" select="document($annotation-url)/annotation"/>-->
    <xsl:variable name="annotation" select="/data/annotations"/>

	<!--
	  - These keys allow input and ouput ports for flows to be found easily.
	  -->
	<xsl:key name="flow-from-port" match="//outputPort" use="flow/@ref"/>
	<xsl:key name="flow-to-port" match="//inputPort" use="flow/@ref"/>

	<!--
	  - The root template
	  -->
	<xsl:template match="/">
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="/data">
    	<xsl:apply-templates/>
	</xsl:template>
	<!--
	  - Match against the top-level element in the BPD and output the top level
	  - SVG.
	  -
	  - This template must also output the scripts and then call templates for
	  - other elements in the correct z-order.
	  -->
	<xsl:template match="BusinessProcessDiagram">
		<!-- Calculate the extent of the SVG, make this a little wider than -->
		<!-- width of all the objects. -->
		<xsl:variable name="lane-width" select="max($skin/(lane|system-lane)/header/@width)"/>
		<xsl:variable name="max-x" select="max(//@x) + 80"/>
		<xsl:variable name="view-x" select="-($skin/pool/header/@width + $lane-width)"/>
		<xsl:variable name="view-y">0</xsl:variable>
		<xsl:variable name="view-w" select="$max-x + $skin/pool/header/@width + $lane-width"/>
		<xsl:variable name="view-h" select="pool/dimension/size/@h"/>

		<!-- Output any file comments -->
		<xsl:comment select="$skin/file-comment"/>

        <!-- Output SVG that will call our javascript on loading -->
<!--        <xsl:element name="svg:svg">
            <xsl:if test="${xml-base}">
                <xsl:attribute name="xml:base"><xsl:value-of select="${xml-base}"/></xsl:attribute>
            </xsl:if>
            <xsl:attribute name="width"><xsl:value-of select="${view-w}"/></xsl:attribute>
            <xsl:attribute name="height"><xsl:value-of select="${view-h}"/></xsl:attribute>
            <xsl:attribute name="viewBox"><xsl:value-of select="${view-x}"/> 0 <xsl:value-of select="${view-w}"/><xsl:value-of select="${view-h}"/></xsl:attribute>
            <xsl:attribute name="a3:scriptImplementation">Adobe</xsl:attribute>
            <xsl:attribute name="onload">Flow._init(); TextWrap._init();</xsl:attribute>-->

		<svg:svg
            width="{$view-w}" height="{$view-h}"
			viewBox="{$view-x} 0 {$view-w} {$view-h}"
			a3:scriptImplementation="Adobe"
			onload="Flow._init(); TextWrap._init();">

			<!-- Load all the scripts -->
			<svg:defs>
				<svg:script type="text/ecmascript" xlink:href="{$skin/script-base}Debug.js"
					a3:scriptImplementation="Adobe" />
				<svg:script type="text/ecmascript" xlink:href="{$skin/script-base}SVG.js"
					a3:scriptImplementation="Adobe" />
				<svg:script type="text/ecmascript" xlink:href="{$skin/script-base}AbstractTextWrap.js"
					a3:scriptImplementation="Adobe" />
				<svg:script type="text/ecmascript" xlink:href="{$skin/script-base}TextWrap.js"
					a3:scriptImplementation="Adobe" />
				<svg:script type="text/ecmascript" xlink:href="{$skin/script-base}Vector.js"
					a3:scriptImplementation="Adobe" />
				<svg:script type="text/ecmascript" xlink:href="{$skin/script-base}Flow.js"
					a3:scriptImplementation="Adobe" />
			</svg:defs>

			<!-- If the user wants any special SVG, put it here -->
			<xsl:copy-of select="$skin/before/*"/>

			<!-- Titles can be shown in browser windows -->
			<svg:title><xsl:value-of select="name" /></svg:title>

			<!-- The order of transforming children is important.  We need -->
			<!-- ensure that things are drawn back to front in the right -->
			<!-- order -->
			<xsl:apply-templates select="pool"/>
			<xsl:apply-templates select="flow" mode="halo"/>
			<xsl:apply-templates select="pool/lane/flowObject" mode="halo"/>
			<xsl:apply-templates select="pool/lane/flowObject"/>
			<xsl:apply-templates select="flow"/>

			<!-- If the user wants any special SVG, put it here -->
			<xsl:copy-of select="$skin/after/*"/>
		</svg:svg>
        <!--</xsl:element>-->
	</xsl:template>

	<!--
	  - Draw the pool header.
	  -
	  - This template also draws the lanes and lane headers (calling out to a
	  - child template).  All elements are then rotated and translated so that
	  - the text for the headers is vertical and the lanes are oriented
	  - horizontally.
	  -->
	<xsl:template match="pool">
		<!-- Need to know how wide the lane headers are so we can position -->
		<!-- The pool header in the right place -->
		<xsl:variable name="lane-width" select="max($skin/(lane|system-lane)/header/@width)"/>
		<svg:g class="pool"
			transform="translate({-($skin/pool/header/@width + $lane-width)},{dimension/size/@h}) rotate(-90)">
			<svg:g class="header">
				<svg:title><xsl:value-of select="documentation"/></svg:title>
				<svg:rect x="0" y="0" width="{dimension/size/@h}"
					height="{$skin/pool/header/@width}" style="{$skin/pool/header/@style}" />
				<lombardi:text-wrap class="text"
					x="0" y="0"
					width="{dimension/size/@h}"
					height="{$skin/pool/header/@width}"
					margin="{$skin/pool/header/text/@margin}"
					style="{$skin/pool/header/text/@style}">
					<xsl:value-of select="name[position()=1]" />
				</lombardi:text-wrap>
			</svg:g>
			<xsl:apply-templates select="lane" />
		</svg:g>
	</xsl:template>

	<!--
	  - Output lanes, system lanes and their headers.
	  -
	  - These elements will be rotated and translated by their parent pool.
	  -->
	<xsl:template match="lane">
		<!-- Lanes need to be offset from each other -->
		<xsl:variable name="offset" select="sum(//lane/height) - lombardi:lane-offset(.) - height"/>
		<!-- Select the right skin -->
		<xsl:variable name="skin-data" select="
			if(systemLane='true')
				then $skin/system-lane
				else $skin/lane"/>

		<svg:g class="lane">
			<!-- The lane background -->
			<svg:rect x="{$offset}" y="{$skin/pool/header/@width + $skin-data/header/@width}" width="{height}"
				height="{../dimension/size/@w}" style="{$skin-data/@style}" />
			<!-- The lane header -->
			<svg:g class="header">
				<svg:title><xsl:value-of select="documentation"/></svg:title>
				<svg:rect style="{$skin/lane/header/@style}"
					x="{$offset}" y="{$skin/pool/header/@width}"
					width="{height}" height="{$skin-data/header/@width}"/>

				<lombardi:text-wrap
					x="{$offset}" y="{$skin/pool/header/@width}"
					width="{height}" height="{$skin-data/header/@width}"
					margin="{$skin-data/header/text/@margin}"
					style="{$skin-data/header/text/@style}">
					<xsl:value-of select="name" />
				</lombardi:text-wrap>
			</svg:g>
		</svg:g>
	</xsl:template>

	<!--
	  - Do the flow objects in non halo mode.
	  -->
	<xsl:template match="flowObject">
		<xsl:variable name="skin-data" select="skin:get-skin(.)" />
		<xsl:variable name="x" select="position/location/@x"/>
		<xsl:variable name="y" select="position/location/@y + lombardi:lane-offset(.)"/>

		<svg:g class="{$skin-data/@type}" transform="translate({$x},{$y})">
			<xsl:sequence select="$skin-data/icon/*"/>

			<!-- Print the name if required -->
			<xsl:if test="if($skin-data/name/@visibility-override)
								then $skin-data/name/@visibility-override='true'
								else component/*/nameVisible='true'">
				<lombardi:text-wrap
					x="{$skin-data/name/text/@x}"
					y="{$skin-data/name/text/@y}"
					width="{$skin-data/name/text/@width}"
					height="{$skin-data/name/text/@height}"
					style="{$skin-data/name/text/@style}">
					<xsl:value-of select="name" />
				</lombardi:text-wrap>
			</xsl:if>

			<!-- Do any overlays for activities -->
			<xsl:sequence select="skin:get-loop-overlay(.)/*"/>
			<xsl:sequence select="skin:get-attached-overlay(.)/*"/>
			<xsl:sequence select="skin:get-is-conditional-overlay(.)/*"/>

			<!-- Do any attached events -->
			<xsl:apply-templates select="attachedEvent"/>
		</svg:g>
	</xsl:template>

	<!--
	  - Do flow objects' halos.
	  -->
	<xsl:template match="flowObject" mode="halo">
		<!-- Check to see if there is a halo for the object -->
		<xsl:variable name="halo" select="$annotation/halo[@flow-object-id=current()/@id]"/>
		<xsl:if test="$halo">
			<xsl:variable name="skin-data" select="skin:get-skin(.)" />
			<xsl:variable name="x" select="position/location/@x"/>
			<xsl:variable name="y" select="position/location/@y + lombardi:lane-offset(.)"/>

			<!-- For halos simply render the images for the activity and -->
			<!-- attached events into a group which has styles applied by -->
			<!-- the skin -->
			<svg:g class="halo"
				transform="translate({$x},{$y})">
				<svg:g>
					<!-- Copy over attributes from the halo skin -->
					<xsl:copy-of select="$skin-data/halo/@*"/>
					<!-- Adobe SVG won't apply filters properly to svg icons -->
					<!-- Consequently we allow special silhouettes to be -->
					<!-- at the user's request -->
					<xsl:sequence select="if($skin-data/silhouette)
						then $skin-data/silhouette/*
						else $skin-data/icon/*"/>
					<xsl:apply-templates select="attachedEvent" mode="halo"/>
				</svg:g>
				<!-- Add halo text -->
				<lombardi:text-wrap
					x="{$skin-data/halo/text/@x}"
					y="{$skin-data/halo/text/@y}"
					width="{$skin-data/halo/text/@width}"
					height="{$skin-data/halo/text/@height}"
					style="{$skin-data/halo/text/@style}">
					<xsl:value-of select="$halo" />
				</lombardi:text-wrap>
			</svg:g>
		</xsl:if>
	</xsl:template>

	<!--
	  - Draw the icon for an attached event
	  -->
	<xsl:template match="attachedEvent">
		<xsl:variable name="skin-data" select="skin:get-skin(.)" />
		<xsl:variable name="port" select="lombardi:get-port-info(..,positionId)"/>
		<xsl:variable name="x" select="$port/location/@x - $skin-data/attachment/@x"/>
		<xsl:variable name="y" select="$port/location/@y - $skin-data/attachment/@y"/>
		<svg:g class="attachedEvent" transform="translate({$x},{$y})">
			<xsl:sequence select="$skin-data/icon/*"/>
		</svg:g>
	</xsl:template>

	<!--
	  - Draw the icon for an attached event
	  -->
	<xsl:template match="attachedEvent" mode="halo">
		<xsl:variable name="skin-data" select="skin:get-skin(.)" />
		<xsl:variable name="port" select="lombardi:get-port-info(..,positionId)"/>
		<xsl:variable name="x" select="$port/location/@x - $skin-data/attachment/@x"/>
		<xsl:variable name="y" select="$port/location/@y - $skin-data/attachment/@y"/>
		<svg:g class="attachedEvent" transform="translate({$x},{$y})">
			<!-- Adobe SVG won't apply filters properly to svg icons. -->
			<!-- Consequently we allow special silhouettes to be -->
			<!-- at the user's request -->
			<xsl:sequence select="if($skin-data/silhouette)
				then $skin-data/silhouette/*
				else $skin-data/icon/*"/>
		</svg:g>
	</xsl:template>

	<!--
	  - Draw the flow
	  -->
	<xsl:template match="flow">
		<!-- Get the port info for the end points of the flow -->
		<xsl:variable name="from-port" select="key('flow-from-port',@id)"/>
		<xsl:variable name="to-port" select="key('flow-to-port',@id)"/>
		<xsl:variable name="from" select="lombardi:get-port-info-absolute($from-port/..,$from-port/positionId)"/>
		<xsl:variable name="to" select="lombardi:get-port-info-absolute($to-port/..,$to-port/positionId)"/>
		<svg:g class="flow">
			<!-- Flows will be routed in Javascript on the client -->
			<!-- Need to give end point locations and normals and any -->
			<!-- control points -->
			<lombardi:flow>
				<!-- Copy over attributes from the skin -->
				<xsl:copy-of select="$skin/flow/@*"/>
				<lombardi:from><xsl:copy-of select="$from/*" /></lombardi:from>
				<xsl:for-each select="controlPoint">
					<lombardi:controlPoint x="{x}" y="{y + lombardi:lane-offset(.)}" />
				</xsl:for-each>
				<lombardi:to><xsl:copy-of select="$to/*" /></lombardi:to>
			</lombardi:flow>

			<!-- Some flows have markers to indicate expressions -->
			<xsl:if test="connection/SequenceFlow/condition/expression">
				<svg:g
					transform="
						matrix({-$from/lombardi:normal/@y} {$from/lombardi:normal/@x} {-$from/lombardi:normal/@x} {-$from/lombardi:normal/@y}
							   {$from/lombardi:location/@x} {$from/lombardi:location/@y})">
					<xsl:sequence select="$skin/flow/expression/*"/>
				</svg:g>
			</xsl:if>

			<!-- All flows have end arrows -->
			<svg:g
				transform="
					matrix({-$to/lombardi:normal/@y} {$to/lombardi:normal/@x} {-$to/lombardi:normal/@x} {-$to/lombardi:normal/@y}
						   {$to/lombardi:location/@x} {$to/lombardi:location/@y})">
				<xsl:sequence select="$skin/flow/end-arrow/*"/>
			</svg:g>

			<!-- Some flows have text on them.  The text will be positioned -->
			<!-- by the javascript on the client which will work out the mid -->
			<!-- point of the text or it will draw the text along the flow -->
			<!-- path -->
			<xsl:if test="nameVisible='true'">
				<lombardi:text-wrap
					x="{$skin/flow/text/@x}" y="{$skin/flow/text/@y}"
					width="{$skin/flow/text/@width}" height="{$skin/flow/text/@height}"
					style="{$skin/flow/text/@style}">
					<xsl:value-of select="name" />
				</lombardi:text-wrap>
			</xsl:if>
		</svg:g>
	</xsl:template>

	<!--
	  - Draw the flow
	  -->
	<xsl:template match="flow" mode="halo">
		<!-- Check to see if there is a halo for the object -->
		<xsl:variable name="halo" select="$annotation/halo[@flow-object-id=current()/@id]"/>
		<xsl:if test="$halo">
			<!-- Get the port info for the end points of the flow -->
			<xsl:variable name="from-port" select="key('flow-from-port',@id)"/>
			<xsl:variable name="to-port" select="key('flow-to-port',@id)"/>
			<xsl:variable name="from" select="lombardi:get-port-info-absolute($from-port/..,$from-port/positionId)"/>
			<xsl:variable name="to" select="lombardi:get-port-info-absolute($to-port/..,$to-port/positionId)"/>
			<svg:g class="halo">
				<!-- Flows will be routed in Javascript on the client -->
				<!-- Need to give end point locations and normals and any -->
				<!-- control points -->
				<lombardi:flow>
					<!-- Copy over attributes from the skin -->
					<xsl:copy-of select="$skin/flowhalo/@*"/>
					<lombardi:from><xsl:copy-of select="$from/*" /></lombardi:from>
					<xsl:for-each select="controlPoint">
						<lombardi:controlPoint x="{x}" y="{y + lombardi:lane-offset(.)}" />
					</xsl:for-each>
					<lombardi:to><xsl:copy-of select="$to/*" /></lombardi:to>
				</lombardi:flow>
			</svg:g>
		</xsl:if>
	</xsl:template>

	<!--
	  - Everything else in the BPD should be ruthlessly eaten.
	  -->
	<xsl:template match="*|@*|text()"></xsl:template>

	<!--
	  - Get the information about a port.
	  -
	  - The information contains the location and normal of the port.  This is
	  - relative the the object that owns the port.
	  -
	  - @param reference the flow object or attached event that owns the port
	  - @param position-id the name of the port
	  - @return a 'port' element with children 'location' and 'normal'
	  -->
	<xsl:function name="lombardi:get-port-info">
		<xsl:param name="reference"/>
		<xsl:param name="position-id"/>

		<xsl:variable name="pos-id">
			<xsl:choose>
				<xsl:when test="name($reference)='attachedEvent'">
					<!-- For attachedEvents, we use the center opposite -->
					<!-- the parent port -->
					<xsl:value-of select="replace(
						$reference/positionId,
						'Top$|Left$|Center$|Right$|Bottom$',
						'Center')"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$position-id"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="skin-data"
			select="skin:get-skin($reference)/ports/port[@name=$pos-id]"/>

		<port>
			<location x="{$skin-data/@x}" y="{$skin-data/@y}"/>
			<xsl:copy-of select="$skin-data/lombardi:normal"/>
		</port>
	</xsl:function>

	<!--
	  - Get the information about a port.
	  -
	  - This function is very similar to the previous 'get-port-info' function,
	  - however, the location is absolute, rather than relative to the reference
	  -
	  - @param reference the flow object or attached event that owns the port
	  - @param position-id the name of the port
	  - @return a 'port' element with children 'location' and 'normal'
	  -->
	<xsl:function name="lombardi:get-port-info-absolute">
		<xsl:param name="reference"/>
		<xsl:param name="position-id"/>

		<xsl:variable name="port-info" select="lombardi:get-port-info($reference,$position-id)"/>
		<port>
			<xsl:sequence select="$port-info/lombardi:normal"/>
			<xsl:choose>
				<xsl:when test="name($reference)='attachedEvent'">
					<xsl:variable name="attachment" select="skin:get-skin($reference)/attachment"/>
					<xsl:variable name="parent-port"
						select="skin:get-skin($reference/..)/ports/port[@name=$reference/positionId]"/>
					<lombardi:location
						x="{$reference/../position/location/@x + $parent-port/@x + $port-info/location/@x - $attachment/@x}"
						y="{$reference/../position/location/@y + $parent-port/@y + $port-info/location/@y - $attachment/@y + lombardi:lane-offset($reference)}"/>
				</xsl:when>
				<xsl:otherwise>
					<lombardi:location
						x="{$reference/position/location/@x + $port-info/location/@x}"
						y="{$reference/position/location/@y + $port-info/location/@y + lombardi:lane-offset($reference)}"/>
				</xsl:otherwise>
			</xsl:choose>
		</port>
	</xsl:function>

	<!--
	  - In the BPD objects' y coordinates are given relative to their lane.
	  - This function calculates th offset that the owning lane provides.
	  -
	  - @param reference the object to get the offset for
	  - @return a number, the offset
	  -->
	<xsl:function name="lombardi:lane-offset">
		<xsl:param name="reference"/>

		<!-- Get the owning lane -->
		<xsl:variable name="lane" select="if(name($reference)='controlPoint')
			then $reference/ancestor::BusinessProcessDiagram/pool/lane[@id=$reference/laneObjectId/BpmnObjectId/@id]
			else $reference/ancestor-or-self::lane"/>
		<!-- Sum the heights of older children -->
		<xsl:value-of select="sum( $lane/preceding-sibling::lane/height )"/>
	</xsl:function>
</xsl:stylesheet>
